import os

import cerberus
from cerberus.solvers import Solver, CodeInput

import numpy as np

cerberus.LOG.set_verbosity(1)

# --- Initialize solvers

sss_input = CodeInput("./Serpent.inp")
sss_path = os.environ.get("SERPENT_EXE_PATH")
sss = Solver("Serpent", sss_path, ["-omp", "8", "-port"])
sss.input = sss_input
sss.initialize()

# --- Get transferrable for keff

ov_keff = sss.get_output_variable("sss_ov_IMP_KEFF")
ov_keff_unc = sss.get_output_variable("sss_ov_IMP_KEFF_rel_unc")

# --- Values for transformation (same number of values as was given in input)

iv_transformation_xyz = sss.get_input_variable("sss_iv_trans_my_trans")

# --- If we send 1 for this, Serpent will plot geometry

iv_plot = sss.get_input_variable("sss_iv_plot_geometry")

# --- Do a couple of iterations

n_iter = 10
z_now = 50.0
keff_prev = 2.0 # Initially, system is supercritical
delta = 20.0

for cur_iter in range(n_iter):

    # --- Run Serpent calculation

    sss.solve()

    # --- Get new values from Serpent

    ov_keff.communicate()
    ov_keff_unc.communicate()

    keff = ov_keff.value_vec[0]
    keff_unc = ov_keff_unc.value_vec[0]

    print(f"\nIteration {cur_iter+1}:")
    print(f"Keff is {keff:.5f}, relative uncertainty is {keff_unc:.3f} with boundary at {z_now:.1f} cm.")

    # --- Move boundary by some distance to some direction

    if keff < 1:

        if keff_prev > 1:
            # --- keff changed sign, lets halve the delta
            delta = delta/2

        # --- Push boron carbide out of core
        z_now = z_now + delta
    else:
        if keff_prev < 1:
            # --- keff changed sign, lets halve the delta
            delta = delta/2

        # --- Pull boron carbide into the core
        z_now = z_now - delta

    # --- Send new value for transformation

    iv_transformation_xyz.set_and_send((0, 0, z_now))

    # --- Plot geometry

    iv_plot.set_and_send(1)

    # --- Remember previous keff

    keff_prev = keff